use serde::Serialize;
use thiserror::Error;

#[derive(Debug, Error, Serialize)]
pub enum AppError {
    #[error("Bluetooth not ready")]
    BluetoothNotReady,
    #[error("No Bluetooth adapter found")]
    BluetoothAdapterNotFound,
    #[error("No available Bluetooth adapter, maybe not powered on")]
    NoAvailableBluetoothAdapter,
    #[error("No connected peripheral")]
    NoConnectedPeripheral,
    #[error("Unable to start scan devices")]
    UnableToStartScan,
    #[error("Unable to stop scan devices")]
    UnableToStopScan,
    #[error("Unable to retrieve peripherals")]
    UnableToRetrievePeripherals,
    #[error("Unable to retrieve peripheral properties")]
    UnableToRetrievePeripheralProperties,
    #[error("Unable to retrieve peripheral state")]
    UnableToRetrievePeripheralState,
    #[error("Unable to retrieve peripheral services")]
    UnableToRetrievePeripheralServices,
    #[error("Visit data storage failed: {0}")]
    StorageFailure(String),
}
