import { listen, UnlistenFn } from '@tauri-apps/api/event';
import { useSetAtom } from 'jotai';
import { FC, useEffect, useRef } from 'react';
import { BleState } from '../context/EstimContext';
import BleControl from '../page-components/device/BleControl';
import DeviceDetail from '../page-components/device/DeviceDetail';
import DeviceList from '../page-components/device/DeviceList';
import styles from './Device.module.css';

const Device: FC = () => {
  const unlistenBleScanStart = useRef<UnlistenFn | null>(null);
  const unlistenBleScanStop = useRef<UnlistenFn | null>(null);
  const refreshBle = useSetAtom(BleState);

  useEffect(() => {
    (async function () {
      try {
        unlistenBleScanStart.current = await listen('scanning_started', () => {
          refreshBle();
        });
        unlistenBleScanStop.current = await listen('scanning_stopped', () => {
          refreshBle();
        });
      } catch (error) {
        console.error('[listen to scan]', error);
      }
    })();

    return () => {
      unlistenBleScanStart.current?.();
      unlistenBleScanStop.current?.();
    };
  }, []);

  return (
    <div className="workspace horizontal">
      <div className={styles.device_list}>
        <BleControl />
        <DeviceList />
      </div>
      <DeviceDetail />
    </div>
  );
};

export default Device;
